@@ -8,6 +8,6 @@ static inline unsigned char crc7_byte(unsigned char crc, unsigned char data)
 	return crc7_syndrome_table[(crc << 1) ^ data];
 }
 
-extern unsigned char crc7(const unsigned char *buffer, unsigned int len);
+extern unsigned char fio_crc7(const unsigned char *buffer, unsigned int len);
 
 #endif
