@@ -22,6 +22,8 @@ enum {
 	FD_COMPRESS,
 	FD_STEADYSTATE,
 	FD_HELPERTHREAD,
+	FD_ZBD,
+	FD_SPRANDOM,
 	FD_DEBUG_MAX,
 };
 
