@@ -30,6 +30,71 @@
 
 #include <sys/stat.h>
 
+#ifndef IO_INTEGRITY_CHK_GUARD
+/* flags for integrity meta */
+#define IO_INTEGRITY_CHK_GUARD		(1U << 0) /* enforce guard check */
+#define IO_INTEGRITY_CHK_REFTAG		(1U << 1) /* enforce ref check */
+#define IO_INTEGRITY_CHK_APPTAG		(1U << 2) /* enforce app check */
+#endif /* IO_INTEGRITY_CHK_GUARD */
+
+#ifndef FS_IOC_GETLBMD_CAP
+/* Protection info capability flags */
+#define	LBMD_PI_CAP_INTEGRITY		(1 << 0)
+#define	LBMD_PI_CAP_REFTAG		(1 << 1)
+
+/* Checksum types for Protection Information */
+#define LBMD_PI_CSUM_NONE		0
+#define LBMD_PI_CSUM_IP			1
+#define LBMD_PI_CSUM_CRC16_T10DIF	2
+#define LBMD_PI_CSUM_CRC64_NVME		4
+
+/*
+ * Logical block metadata capability descriptor
+ * If the device does not support metadata, all the fields will be zero.
+ * Applications must check lbmd_flags to determine whether metadata is
+ * supported or not.
+ */
+struct logical_block_metadata_cap {
+	/* Bitmask of logical block metadata capability flags */
+	__u32	lbmd_flags;
+	/*
+	 * The amount of data described by each unit of logical block
+	 * metadata
+	 */
+	__u16	lbmd_interval;
+	/*
+	 * Size in bytes of the logical block metadata associated with each
+	 * interval
+	 */
+	__u8	lbmd_size;
+	/*
+	 * Size in bytes of the opaque block tag associated with each
+	 * interval
+	 */
+	__u8	lbmd_opaque_size;
+	/*
+	 * Offset in bytes of the opaque block tag within the logical block
+	 * metadata
+	 */
+	__u8	lbmd_opaque_offset;
+	/* Size in bytes of the T10 PI tuple associated with each interval */
+	__u8	lbmd_pi_size;
+	/* Offset in bytes of T10 PI tuple within the logical block metadata */
+	__u8	lbmd_pi_offset;
+	/* T10 PI guard tag type */
+	__u8	lbmd_guard_tag_type;
+	/* Size in bytes of the T10 PI application tag */
+	__u8	lbmd_app_tag_size;
+	/* Size in bytes of the T10 PI reference tag */
+	__u8	lbmd_ref_tag_size;
+	/* Size in bytes of the T10 PI storage tag */
+	__u8	lbmd_storage_tag_size;
+	__u8	pad;
+};
+
+#define FS_IOC_GETLBMD_CAP			_IOWR(0x15, 2, struct logical_block_metadata_cap)
+#endif /* FS_IOC_GETLBMD_CAP */
+
 enum uring_cmd_type {
 	FIO_URING_CMD_NVME = 1,
 };
@@ -73,6 +138,7 @@ struct ioring_data {
 
 	struct io_u **io_u_index;
 	char *md_buf;
+	char *pi_attr;
 
 	int *fds;
 
@@ -97,6 +163,10 @@ struct ioring_data {
 	struct nvme_dsm *dsm;
 	uint32_t cdw12_flags[DDIR_RWDIR_CNT];
 	uint8_t write_opcode;
+
+	bool is_uring_cmd_eng;
+
+	struct nvme_cmd_ext_io_opts ext_opts;
 };
 
 struct ioring_options {
@@ -180,7 +250,7 @@ static struct fio_option options[] = {
 		.lname	= "Additional Write commands support (Write Uncorrectable, Write Zeores)",
 		.type	= FIO_OPT_STR,
 		.off1	= offsetof(struct ioring_options, write_mode),
-		.help	= "Issue Write Uncorrectable or Zeroes command instaed of Write command",
+		.help	= "Issue Write Uncorrectable or Zeroes command instead of Write command",
 		.def	= "write",
 		.posval = {
 			  { .ival = "write",
@@ -394,6 +464,25 @@ static int io_uring_enter(struct ioring_data *ld, unsigned int to_submit,
 #define BLOCK_URING_CMD_DISCARD	_IO(0x12, 0)
 #endif
 
+static void fio_ioring_prep_md(struct thread_data *td, struct io_u *io_u)
+{
+	struct ioring_data *ld = td->io_ops_data;
+	struct io_uring_attr_pi *pi_attr = io_u->pi_attr;
+	struct nvme_data *data = FILE_ENG_DATA(io_u->file);
+	struct io_uring_sqe *sqe;
+
+	sqe = &ld->sqes[io_u->index];
+
+	sqe->attr_type_mask = IORING_RW_ATTR_FLAG_PI;
+	sqe->attr_ptr = (__u64)(uintptr_t)pi_attr;
+	pi_attr->addr = (__u64)(uintptr_t)io_u->mmap_data;
+
+	if (pi_attr->flags & IO_INTEGRITY_CHK_REFTAG) {
+		__u64 slba = get_slba(data, io_u->offset);
+		pi_attr->seed = (__u32)slba;
+	}
+}
+
 static int fio_ioring_prep(struct thread_data *td, struct io_u *io_u)
 {
 	struct ioring_data *ld = td->io_ops_data;
@@ -436,6 +525,8 @@ static int fio_ioring_prep(struct thread_data *td, struct io_u *io_u)
 				sqe->len = 1;
 			}
 		}
+		if (o->md_per_io_size)
+			fio_ioring_prep_md(td, io_u);
 		sqe->rw_flags = 0;
 		if (!td->o.odirect && o->uncached)
 			sqe->rw_flags |= RWF_DONTCACHE;
@@ -562,9 +653,26 @@ static int fio_ioring_cmd_prep(struct thread_data *td, struct io_u *io_u)
 			ld->cdw12_flags[io_u->ddir]);
 }
 
+static void fio_ioring_validate_md(struct thread_data *td, struct io_u *io_u)
+{
+	struct nvme_data *data;
+	struct ioring_options *o = td->eo;
+	int ret;
+
+	data = FILE_ENG_DATA(io_u->file);
+	if (data->pi_type && (io_u->ddir == DDIR_READ) && !o->pi_act) {
+		ret = fio_nvme_pi_verify(data, io_u);
+		if (ret)
+			io_u->error = -ret;
+	}
+
+	return;
+}
+
 static struct io_u *fio_ioring_event(struct thread_data *td, int event)
 {
 	struct ioring_data *ld = td->io_ops_data;
+	struct ioring_options *o = td->eo;
 	struct io_uring_cqe *cqe;
 	struct io_u *io_u;
 	unsigned index;
@@ -590,8 +698,13 @@ static struct io_u *fio_ioring_event(struct thread_data *td, int event)
 			io_u->error = -cqe->res;
 		else
 			io_u->resid = io_u->xfer_buflen - cqe->res;
+
+		return io_u;
 	}
 
+	if (o->md_per_io_size)
+		fio_ioring_validate_md(td, io_u);
+
 	return io_u;
 }
 
@@ -674,25 +787,26 @@ static char *fio_ioring_cmd_errdetails(struct thread_data *td,
 	return msg;
 }
 
-static int fio_ioring_cqring_reap(struct thread_data *td, unsigned int events,
-				   unsigned int max)
+static unsigned fio_ioring_cqring_reap(struct thread_data *td, unsigned int max)
 {
 	struct ioring_data *ld = td->io_ops_data;
 	struct io_cq_ring *ring = &ld->cq_ring;
-	unsigned head, reaped = 0;
+	unsigned head = *ring->head;
+	unsigned available = atomic_load_acquire(ring->tail) - head;
 
-	head = *ring->head;
-	do {
-		if (head == atomic_load_acquire(ring->tail))
-			break;
-		reaped++;
-		head++;
-	} while (reaped + events < max);
+	if (!available)
+		return 0;
 
-	if (reaped)
-		atomic_store_release(ring->head, head);
-
-	return reaped;
+	available = min(available, max);
+	/*
+	 * The CQ consumer index is advanced before the CQEs are actually read.
+	 * This is generally unsafe, as it lets the kernel reuse the CQE slots.
+	 * However, the CQ is sized large enough for the maximum iodepth and a
+	 * new SQE won't be submitted until the CQE is processed, so the CQE
+	 * slot won't actually be reused until it has been processed.
+	 */
+	atomic_store_relaxed(ring->head, head + available);
+	return available;
 }
 
 static int fio_ioring_getevents(struct thread_data *td, unsigned int min,
@@ -706,14 +820,15 @@ static int fio_ioring_getevents(struct thread_data *td, unsigned int min,
 	int r;
 
 	ld->cq_ring_off = *ring->head;
-	do {
-		r = fio_ioring_cqring_reap(td, events, max);
+	for (;;) {
+		r = fio_ioring_cqring_reap(td, max - events);
 		if (r) {
 			events += r;
-			max -= r;
+			if (events >= min)
+				return events;
+
 			if (actual_min != 0)
 				actual_min -= r;
-			continue;
 		}
 
 		if (!o->sqpoll_thread) {
@@ -724,23 +839,18 @@ static int fio_ioring_getevents(struct thread_data *td, unsigned int min,
 					continue;
 				r = -errno;
 				td_verror(td, errno, "io_uring_enter");
-				break;
+				return r;
 			}
 		}
-	} while (events < min);
-
-	return r < 0 ? r : events;
+	}
 }
 
 static inline void fio_ioring_cmd_nvme_pi(struct thread_data *td,
 					  struct io_u *io_u)
 {
 	struct ioring_data *ld = td->io_ops_data;
-	struct ioring_options *o = td->eo;
 	struct nvme_uring_cmd *cmd;
 	struct io_uring_sqe *sqe;
-	struct nvme_cmd_ext_io_opts ext_opts = {0};
-	struct nvme_data *data = FILE_ENG_DATA(io_u->file);
 
 	if (io_u->ddir == DDIR_TRIM)
 		return;
@@ -748,15 +858,18 @@ static inline void fio_ioring_cmd_nvme_pi(struct thread_data *td,
 	sqe = &ld->sqes[(io_u->index) << 1];
 	cmd = (struct nvme_uring_cmd *)sqe->cmd;
 
-	if (data->pi_type) {
-		if (o->pi_act)
-			ext_opts.io_flags |= NVME_IO_PRINFO_PRACT;
-		ext_opts.io_flags |= o->prchk;
-		ext_opts.apptag = o->apptag;
-		ext_opts.apptag_mask = o->apptag_mask;
-	}
+	fio_nvme_pi_fill(cmd, io_u, &ld->ext_opts);
+}
 
-	fio_nvme_pi_fill(cmd, io_u, &ext_opts);
+static inline void fio_ioring_setup_pi(struct thread_data *td,
+				      struct io_u *io_u)
+{
+	struct ioring_data *ld = td->io_ops_data;
+
+	if (io_u->ddir == DDIR_TRIM)
+		return;
+
+	fio_nvme_generate_guard(io_u, &ld->ext_opts);
 }
 
 static inline void fio_ioring_cmdprio_prep(struct thread_data *td,
@@ -775,7 +888,7 @@ static enum fio_q_status fio_ioring_queue(struct thread_data *td,
 	struct ioring_data *ld = td->io_ops_data;
 	struct ioring_options *o = td->eo;
 	struct io_sq_ring *ring = &ld->sq_ring;
-	unsigned tail, next_tail;
+	unsigned tail;
 
 	fio_ro_check(td, io_u);
 
@@ -795,20 +908,17 @@ static enum fio_q_status fio_ioring_queue(struct thread_data *td,
 		return FIO_Q_COMPLETED;
 	}
 
-	tail = *ring->tail;
-	next_tail = tail + 1;
-	if (next_tail == atomic_load_relaxed(ring->head))
-		return FIO_Q_BUSY;
-
 	if (ld->cmdprio.mode != CMDPRIO_MODE_NONE)
 		fio_ioring_cmdprio_prep(td, io_u);
 
-	if (!strcmp(td->io_ops->name, "io_uring_cmd") &&
-		o->cmd_type == FIO_URING_CMD_NVME)
+	if (o->cmd_type == FIO_URING_CMD_NVME && ld->is_uring_cmd_eng)
 		fio_ioring_cmd_nvme_pi(td, io_u);
+	else if (o->md_per_io_size)
+		fio_ioring_setup_pi(td, io_u);
 
+	tail = *ring->tail;
 	ring->array[tail & ld->sq_ring_mask] = io_u->index;
-	atomic_store_release(ring->tail, next_tail);
+	atomic_store_release(ring->tail, tail + 1);
 
 	ld->queued++;
 	return FIO_Q_QUEUED;
@@ -888,7 +998,7 @@ static int fio_ioring_commit(struct thread_data *td)
 			continue;
 		} else {
 			if (errno == EAGAIN || errno == EINTR) {
-				ret = fio_ioring_cqring_reap(td, 0, ld->queued);
+				ret = fio_ioring_cqring_reap(td, ld->queued);
 				if (ret)
 					continue;
 				/* Shouldn't happen */
@@ -924,6 +1034,7 @@ static void fio_ioring_cleanup(struct thread_data *td)
 		fio_cmdprio_cleanup(&ld->cmdprio);
 		free(ld->io_u_index);
 		free(ld->md_buf);
+		free(ld->pi_attr);
 		free(ld->iovecs);
 		free(ld->fds);
 		free(ld->dsm);
@@ -1318,6 +1429,37 @@ static void parse_prchk_flags(struct ioring_options *o)
 		o->prchk |= NVME_IO_PRINFO_PRCHK_APP;
 }
 
+static int fio_ioring_cmd_init(struct thread_data *td, struct ioring_data *ld)
+{
+	struct ioring_options *o = td->eo;
+
+	if (td_write(td)) {
+		switch (o->write_mode) {
+		case FIO_URING_CMD_WMODE_UNCOR:
+			ld->write_opcode = nvme_cmd_write_uncor;
+			break;
+		case FIO_URING_CMD_WMODE_ZEROES:
+			ld->write_opcode = nvme_cmd_write_zeroes;
+			if (o->deac)
+				ld->cdw12_flags[DDIR_WRITE] = 1 << 25;
+			break;
+		case FIO_URING_CMD_WMODE_VERIFY:
+			ld->write_opcode = nvme_cmd_verify;
+			break;
+		default:
+			ld->write_opcode = nvme_cmd_write;
+			break;
+		}
+	}
+
+	if (o->readfua)
+		ld->cdw12_flags[DDIR_READ] = 1 << 30;
+	if (o->writefua)
+		ld->cdw12_flags[DDIR_WRITE] = 1 << 30;
+
+	return 0;
+}
+
 static int fio_ioring_init(struct thread_data *td)
 {
 	struct ioring_options *o = td->eo;
@@ -1327,6 +1469,7 @@ static int fio_ioring_init(struct thread_data *td)
 	unsigned int dsm_size;
 	unsigned long long md_size;
 	int ret, i;
+	struct nvme_cmd_ext_io_opts *ext_opts;
 
 	/* sqthread submission requires registered files */
 	if (o->sqpoll_thread)
@@ -1340,6 +1483,8 @@ static int fio_ioring_init(struct thread_data *td)
 
 	ld = calloc(1, sizeof(*ld));
 
+	ld->is_uring_cmd_eng = (td->io_ops->prep == fio_ioring_cmd_prep);
+
 	/*
 	 * The internal io_uring queue depth must be a power-of-2, as that's
 	 * how the ring interface works. So round that up, in case the user
@@ -1351,12 +1496,21 @@ static int fio_ioring_init(struct thread_data *td)
 	/* io_u index */
 	ld->io_u_index = calloc(td->o.iodepth, sizeof(struct io_u *));
 
+	if (!ld->is_uring_cmd_eng && o->md_per_io_size) {
+		if (o->apptag_mask != 0xffff) {
+			log_err("fio: io_uring with metadata requires an apptag_mask of 0xffff\n");
+			free(ld->io_u_index);
+			free(ld);
+			return 1;
+		}
+	}
+
 	/*
-	 * metadata buffer for nvme command.
+	 * metadata buffer
 	 * We are only supporting iomem=malloc / mem=malloc as of now.
 	 */
-	if (!strcmp(td->io_ops->name, "io_uring_cmd") &&
-	    (o->cmd_type == FIO_URING_CMD_NVME) && o->md_per_io_size) {
+	if (o->md_per_io_size && (!ld->is_uring_cmd_eng ||
+	    (ld->is_uring_cmd_eng && o->cmd_type == FIO_URING_CMD_NVME))) {
 		md_size = (unsigned long long) o->md_per_io_size
 				* (unsigned long long) td->o.iodepth;
 		md_size += page_mask + td->o.mem_align;
@@ -1364,11 +1518,29 @@ static int fio_ioring_init(struct thread_data *td)
 			md_size += td->o.mem_align - page_size;
 		ld->md_buf = malloc(md_size);
 		if (!ld->md_buf) {
+			free(ld->io_u_index);
 			free(ld);
 			return 1;
 		}
+
+		if (!ld->is_uring_cmd_eng) {
+			ld->pi_attr = calloc(ld->iodepth, sizeof(struct io_uring_attr_pi));
+			if (!ld->pi_attr) {
+				free(ld->io_u_index);
+				free(ld->md_buf);
+				free(ld);
+				return 1;
+			}
+		}
+
 	}
 	parse_prchk_flags(o);
+	ext_opts = &ld->ext_opts;
+	if (o->pi_act)
+		ext_opts->io_flags |= NVME_IO_PRINFO_PRACT;
+	ext_opts->io_flags |= o->prchk;
+	ext_opts->apptag = o->apptag;
+	ext_opts->apptag_mask = o->apptag_mask;
 
 	ld->iovecs = calloc(ld->iodepth, sizeof(struct iovec));
 
@@ -1384,12 +1556,12 @@ static int fio_ioring_init(struct thread_data *td)
 	 * For io_uring_cmd, trims are async operations unless we are operating
 	 * in zbd mode where trim means zone reset.
 	 */
-	if (!strcmp(td->io_ops->name, "io_uring_cmd") && td_trim(td) &&
-	    td->o.zone_mode == ZONE_MODE_ZBD) {
+	if (td_trim(td) && td->o.zone_mode == ZONE_MODE_ZBD &&
+	    ld->is_uring_cmd_eng) {
 		td->io_ops->flags |= FIO_ASYNCIO_SYNC_TRIM;
 	} else {
-		dsm_size = sizeof(*ld->dsm) +
-			td->o.num_range * sizeof(struct nvme_dsm_range);
+		dsm_size = sizeof(*ld->dsm);
+		dsm_size += td->o.num_range * sizeof(struct nvme_dsm_range);
 		ld->dsm = calloc(td->o.iodepth, dsm_size);
 		ptr = ld->dsm;
 		for (i = 0; i < td->o.iodepth; i++) {
@@ -1399,32 +1571,8 @@ static int fio_ioring_init(struct thread_data *td)
 		}
 	}
 
-	if (!strcmp(td->io_ops->name, "io_uring_cmd")) {
-		if (td_write(td)) {
-			switch (o->write_mode) {
-			case FIO_URING_CMD_WMODE_UNCOR:
-				ld->write_opcode = nvme_cmd_write_uncor;
-				break;
-			case FIO_URING_CMD_WMODE_ZEROES:
-				ld->write_opcode = nvme_cmd_write_zeroes;
-				if (o->deac)
-					ld->cdw12_flags[DDIR_WRITE] = 1 << 25;
-				break;
-			case FIO_URING_CMD_WMODE_VERIFY:
-				ld->write_opcode = nvme_cmd_verify;
-				break;
-			default:
-				ld->write_opcode = nvme_cmd_write;
-				break;
-			}
-		}
-
-		if (o->readfua)
-			ld->cdw12_flags[DDIR_READ] = 1 << 30;
-		if (o->writefua)
-			ld->cdw12_flags[DDIR_WRITE] = 1 << 30;
-	}
-
+	if (ld->is_uring_cmd_eng)
+		return fio_ioring_cmd_init(td, ld);
 	return 0;
 }
 
@@ -1433,22 +1581,39 @@ static int fio_ioring_io_u_init(struct thread_data *td, struct io_u *io_u)
 	struct ioring_data *ld = td->io_ops_data;
 	struct ioring_options *o = td->eo;
 	struct nvme_pi_data *pi_data;
-	char *p;
+	char *p, *q;
 
 	ld->io_u_index[io_u->index] = io_u;
 
-	if (!strcmp(td->io_ops->name, "io_uring_cmd")) {
-		p = PTR_ALIGN(ld->md_buf, page_mask) + td->o.mem_align;
-		p += o->md_per_io_size * io_u->index;
-		io_u->mmap_data = p;
+	p = PTR_ALIGN(ld->md_buf, page_mask) + td->o.mem_align;
+	p += o->md_per_io_size * io_u->index;
+	io_u->mmap_data = p;
 
-		if (!o->pi_act) {
-			pi_data = calloc(1, sizeof(*pi_data));
-			pi_data->io_flags |= o->prchk;
-			pi_data->apptag_mask = o->apptag_mask;
-			pi_data->apptag = o->apptag;
-			io_u->engine_data = pi_data;
-		}
+	if (ld->pi_attr) {
+		struct io_uring_attr_pi *pi_attr;
+
+		q = ld->pi_attr;
+		q += (sizeof(struct io_uring_attr_pi) * io_u->index);
+		io_u->pi_attr = q;
+
+		pi_attr = io_u->pi_attr;
+		pi_attr->len = o->md_per_io_size;
+		pi_attr->app_tag = o->apptag;
+		pi_attr->flags = 0;
+		if (strstr(o->pi_chk, "GUARD") != NULL)
+			pi_attr->flags |= IO_INTEGRITY_CHK_GUARD;
+		if (strstr(o->pi_chk, "REFTAG") != NULL)
+			pi_attr->flags |= IO_INTEGRITY_CHK_REFTAG;
+		if (strstr(o->pi_chk, "APPTAG") != NULL)
+			pi_attr->flags |= IO_INTEGRITY_CHK_APPTAG;
+	}
+
+	if (!o->pi_act) {
+		pi_data = calloc(1, sizeof(*pi_data));
+		pi_data->io_flags |= o->prchk;
+		pi_data->apptag_mask = o->apptag_mask;
+		pi_data->apptag = o->apptag;
+		io_u->engine_data = pi_data;
 	}
 
 	return 0;
@@ -1456,15 +1621,91 @@ static int fio_ioring_io_u_init(struct thread_data *td, struct io_u *io_u)
 
 static void fio_ioring_io_u_free(struct thread_data *td, struct io_u *io_u)
 {
-	struct ioring_options *o = td->eo;
-	struct nvme_pi *pi;
+	struct nvme_pi *pi = io_u->engine_data;
 
-	if (!strcmp(td->io_ops->name, "io_uring_cmd") &&
-	    (o->cmd_type == FIO_URING_CMD_NVME)) {
-		pi = io_u->engine_data;
-		free(pi);
-		io_u->engine_data = NULL;
+	free(pi);
+	io_u->engine_data = NULL;
+}
+
+static int fio_get_pi_info(struct fio_file *f, struct nvme_data *data)
+{
+	struct logical_block_metadata_cap md_cap;
+	int ret;
+	int fd, err = 0;
+
+	fd = open(f->file_name, O_RDONLY);
+	if (fd < 0)
+		return -errno;
+
+	ret = ioctl(fd, FS_IOC_GETLBMD_CAP, &md_cap);
+	if (ret < 0) {
+		err = -errno;
+		log_err("%s: failed to query protection information capabilities; error %d\n", f->file_name, errno);
+		goto out;
 	}
+
+	if (!(md_cap.lbmd_flags & LBMD_PI_CAP_INTEGRITY)) {
+		log_err("%s: Protection information not supported\n", f->file_name);
+		err = -ENOTSUP;
+		goto out;
+	}
+
+	/* Currently we don't support storage tags */
+	if (md_cap.lbmd_storage_tag_size) {
+		log_err("%s: Storage tag not supported\n", f->file_name);
+		err = -ENOTSUP;
+		goto out;
+	}
+
+	data->lba_size = md_cap.lbmd_interval;
+	data->lba_shift = ilog2(data->lba_size);
+	data->ms = md_cap.lbmd_size;
+	data->pi_size = md_cap.lbmd_pi_size;
+	data->pi_loc = !(md_cap.lbmd_pi_offset);
+
+	/* Assume Type 1 PI if reference tags supported */
+	if (md_cap.lbmd_flags & LBMD_PI_CAP_REFTAG)
+		data->pi_type = NVME_NS_DPS_PI_TYPE1;
+	else
+		data->pi_type = NVME_NS_DPS_PI_TYPE3;
+
+	switch (md_cap.lbmd_guard_tag_type) {
+	case LBMD_PI_CSUM_CRC16_T10DIF:
+		data->guard_type = NVME_NVM_NS_16B_GUARD;
+		break;
+	case LBMD_PI_CSUM_CRC64_NVME:
+		data->guard_type = NVME_NVM_NS_64B_GUARD;
+		break;
+	default:
+		log_err("%s: unsupported checksum type %d\n", f->file_name,
+				md_cap.lbmd_guard_tag_type);
+		err = -ENOTSUP;
+		goto out;
+	}
+
+out:
+	close(fd);
+	return err;
+}
+
+static inline int fio_ioring_open_file_md(struct thread_data *td, struct fio_file *f)
+{
+	int ret = 0;
+	struct nvme_data *data = NULL;
+
+	data = FILE_ENG_DATA(f);
+	if (data == NULL) {
+		data = calloc(1, sizeof(struct nvme_data));
+		ret = fio_get_pi_info(f, data);
+		if (ret) {
+			free(data);
+			return ret;
+		}
+
+		FILE_SET_ENG_DATA(f, data);
+	}
+
+	return ret;
 }
 
 static int fio_ioring_open_file(struct thread_data *td, struct fio_file *f)
@@ -1472,6 +1713,17 @@ static int fio_ioring_open_file(struct thread_data *td, struct fio_file *f)
 	struct ioring_data *ld = td->io_ops_data;
 	struct ioring_options *o = td->eo;
 
+	if (o->md_per_io_size) {
+		/*
+		 * This will be a no-op when called by the io_uring_cmd
+		 * ioengine because engine data has already been collected by
+		 * the time this call is made
+		 */
+		int ret = fio_ioring_open_file_md(td, f);
+		if (ret)
+			return ret;
+	}
+
 	if (!ld || !o->registerfiles)
 		return generic_open_file(td, f);
 
@@ -1479,88 +1731,104 @@ static int fio_ioring_open_file(struct thread_data *td, struct fio_file *f)
 	return 0;
 }
 
+static int verify_params(struct thread_data *td, struct nvme_data *data,
+			 struct fio_file *f, enum fio_ddir ddir)
+{
+	struct ioring_options *o = td->eo;
+	unsigned int lba_size;
+
+	lba_size = data->lba_ext ? data->lba_ext : data->lba_size;
+	if (td->o.min_bs[ddir] % lba_size || td->o.max_bs[ddir] % lba_size) {
+		if (data->lba_ext) {
+			log_err("%s: block size must be a multiple of %u "
+				"(LBA data size + Metadata size)\n", f->file_name, lba_size);
+			if (td->o.min_bs[ddir] == td->o.max_bs[ddir] &&
+			    !(td->o.min_bs[ddir] % data->lba_size)) {
+				/* fixed block size is actually a multiple of LBA data size */
+				unsigned long long suggestion = lba_size *
+					(td->o.min_bs[ddir] / data->lba_size);
+				log_err("Did you mean to use a block size of %llu?\n", suggestion);
+			}
+		} else {
+			log_err("%s: block size must be a multiple of LBA data size\n",
+				f->file_name);
+		}
+		td_verror(td, EINVAL, "fio_ioring_cmd_open_file");
+		return 1;
+	}
+	if (data->ms && !data->lba_ext && ddir != DDIR_TRIM &&
+	    (o->md_per_io_size < ((td->o.max_bs[ddir] / data->lba_size) * data->ms))) {
+		log_err("%s: md_per_io_size should be at least %llu bytes\n",
+			f->file_name,
+			((td->o.max_bs[ddir] / data->lba_size) * data->ms));
+		td_verror(td, EINVAL, "fio_ioring_cmd_open_file");
+		return 1;
+	}
+
+	return 0;
+}
+
+static int fio_ioring_open_nvme(struct thread_data *td, struct fio_file *f)
+{
+	struct ioring_options *o = td->eo;
+	struct nvme_data *data = NULL;
+	__u64 nlba = 0;
+	int ret;
+
+	/* Store the namespace-id and lba size. */
+	data = FILE_ENG_DATA(f);
+	if (data == NULL) {
+		data = calloc(1, sizeof(struct nvme_data));
+		ret = fio_nvme_get_info(f, &nlba, o->pi_act, data);
+		if (ret) {
+			free(data);
+			return ret;
+		}
+
+		FILE_SET_ENG_DATA(f, data);
+	}
+
+	for_each_rw_ddir(ddir) {
+		ret = verify_params(td, data, f, ddir);
+		if (ret)
+			return ret;
+	}
+
+	/*
+	 * For extended logical block sizes we cannot use verify when
+	 * end to end data protection checks are enabled, as the PI
+	 * section of data buffer conflicts with verify.
+	 */
+	if (data->ms && data->pi_type && data->lba_ext &&
+	    td->o.verify != VERIFY_NONE) {
+		log_err("%s: for extended LBA, verify cannot be used when E2E "
+			"data protection is enabled\n", f->file_name);
+		td_verror(td, EINVAL, "fio_ioring_cmd_open_file");
+		return 1;
+	}
+
+	if (o->write_mode != FIO_URING_CMD_WMODE_WRITE && !td_write(td)) {
+		log_err("%s: 'readwrite=|rw=' has no write\n", f->file_name);
+		td_verror(td, EINVAL, "fio_ioring_cmd_open_file");
+		return 1;
+	}
+
+	return 0;
+}
+
 static int fio_ioring_cmd_open_file(struct thread_data *td, struct fio_file *f)
 {
-	struct ioring_data *ld = td->io_ops_data;
 	struct ioring_options *o = td->eo;
 
 	if (o->cmd_type == FIO_URING_CMD_NVME) {
-		struct nvme_data *data = NULL;
-		unsigned int lba_size = 0;
-		__u64 nlba = 0;
 		int ret;
 
-		/* Store the namespace-id and lba size. */
-		data = FILE_ENG_DATA(f);
-		if (data == NULL) {
-			data = calloc(1, sizeof(struct nvme_data));
-			ret = fio_nvme_get_info(f, &nlba, o->pi_act, data);
-			if (ret) {
-				free(data);
-				return ret;
-			}
-
-			FILE_SET_ENG_DATA(f, data);
-		}
-
-		lba_size = data->lba_ext ? data->lba_ext : data->lba_size;
-
-		for_each_rw_ddir(ddir) {
-			if (td->o.min_bs[ddir] % lba_size || td->o.max_bs[ddir] % lba_size) {
-				if (data->lba_ext) {
-					log_err("%s: block size must be a multiple of %u "
-						"(LBA data size + Metadata size)\n", f->file_name, lba_size);
-					if (td->o.min_bs[ddir] == td->o.max_bs[ddir] &&
-					    !(td->o.min_bs[ddir] % data->lba_size)) {
-						/* fixed block size is actually a multiple of LBA data size */
-						unsigned long long suggestion = lba_size *
-							(td->o.min_bs[ddir] / data->lba_size);
-						log_err("Did you mean to use a block size of %llu?\n", suggestion);
-					}
-				} else {
-					log_err("%s: block size must be a multiple of LBA data size\n",
-						f->file_name);
-				}
-				td_verror(td, EINVAL, "fio_ioring_cmd_open_file");
-				return 1;
-			}
-			if (data->ms && !data->lba_ext && ddir != DDIR_TRIM &&
-			    (o->md_per_io_size < ((td->o.max_bs[ddir] / data->lba_size) *
-						  data->ms))) {
-				log_err("%s: md_per_io_size should be at least %llu bytes\n",
-					f->file_name,
-					((td->o.max_bs[ddir] / data->lba_size) * data->ms));
-				td_verror(td, EINVAL, "fio_ioring_cmd_open_file");
-				return 1;
-			}
-                }
-
-		/*
-		 * For extended logical block sizes we cannot use verify when
-		 * end to end data protection checks are enabled, as the PI
-		 * section of data buffer conflicts with verify.
-		 */
-		if (data->ms && data->pi_type && data->lba_ext &&
-		    td->o.verify != VERIFY_NONE) {
-			log_err("%s: for extended LBA, verify cannot be used when E2E data protection is enabled\n",
-				f->file_name);
-			td_verror(td, EINVAL, "fio_ioring_cmd_open_file");
-			return 1;
-		}
-
-		if (o->write_mode != FIO_URING_CMD_WMODE_WRITE &&
-		    !td_write(td)) {
-			log_err("%s: 'readwrite=|rw=' has no write\n",
-					f->file_name);
-			td_verror(td, EINVAL, "fio_ioring_cmd_open_file");
-			return 1;
-		}
+		ret = fio_ioring_open_nvme(td, f);
+		if (ret)
+			return ret;
 	}
-	if (!ld || !o->registerfiles)
-		return generic_open_file(td, f);
 
-	f->fd = ld->fds[f->engine_pos];
-	return 0;
+	return fio_ioring_open_file(td, f);
 }
 
 static int fio_ioring_close_file(struct thread_data *td, struct fio_file *f)
@@ -1578,7 +1846,6 @@ static int fio_ioring_close_file(struct thread_data *td, struct fio_file *f)
 static int fio_ioring_cmd_close_file(struct thread_data *td,
 				     struct fio_file *f)
 {
-	struct ioring_data *ld = td->io_ops_data;
 	struct ioring_options *o = td->eo;
 
 	if (o->cmd_type == FIO_URING_CMD_NVME) {
@@ -1587,11 +1854,8 @@ static int fio_ioring_cmd_close_file(struct thread_data *td,
 		FILE_SET_ENG_DATA(f, NULL);
 		free(data);
 	}
-	if (!ld || !o->registerfiles)
-		return generic_close_file(td, f);
 
-	f->fd = -1;
-	return 0;
+	return fio_ioring_close_file(td, f);
 }
 
 static int fio_ioring_cmd_get_file_size(struct thread_data *td,
@@ -1614,7 +1878,10 @@ static int fio_ioring_cmd_get_file_size(struct thread_data *td,
 			return ret;
 		}
 
-		f->real_file_size = data->lba_size * nlba;
+		if (data->lba_ext)
+			f->real_file_size = data->lba_ext * nlba;
+		else
+			f->real_file_size = data->lba_size * nlba;
 		fio_file_set_size_known(f);
 
 		FILE_SET_ENG_DATA(f, data);
@@ -1684,6 +1951,7 @@ static struct ioengine_ops ioengine_uring = {
 	.init			= fio_ioring_init,
 	.post_init		= fio_ioring_post_init,
 	.io_u_init		= fio_ioring_io_u_init,
+	.io_u_free		= fio_ioring_io_u_free,
 	.prep			= fio_ioring_prep,
 	.queue			= fio_ioring_queue,
 	.commit			= fio_ioring_commit,
