@@ -13,4 +13,8 @@ typedef int bool;
 #include <stdbool.h> /* IWYU pragma: export */
 #endif
 
+#if !defined(CONFIG_HAVE_KERNEL_RWF_T)
+typedef int __kernel_rwf_t;
+#endif
+
 #endif
